@@ -11,13 +11,14 @@ use POSIX;
 
 use Error qw(:try);
 use Sys::Hostname;
-use TheSchwartz;
 
 use Helios;
 use Helios::Error;
 use Helios::LogEntry::Levels qw(:all);
+use Helios::TheSchwartz;
+use Helios::Config;
 
-our $VERSION = '2.41';
+our $VERSION = '2.60';
 
 =head1 NAME
 
@@ -227,7 +228,9 @@ our %DEFAULTS = (
     MASTER_LAUNCH_INTERVAL => 1,
     ZERO_LAUNCH_INTERVAL => 10,
     ZERO_SLEEP_INTERVAL => 10,
-    REGISTRATION_INTERVAL => 60
+    REGISTRATION_INTERVAL => 60,
+    WORKER_BLITZ_FACTOR => 1,
+    WORKER_MAX_TTL_WAIT_INTERVAL => 20
 );
 our $CLEAN_SHUTDOWN = 1;				# used to determine if we should remove the PID file or not (at least for now)
 
@@ -248,11 +251,15 @@ our $SAFE_MODE_RETRIES = 5;				# SAFE MODE support; number of times to retry
 our $ZERO_SLEEP_INTERVAL;				# to reduce needless checking of the database
 our $ZERO_SLEEP_LOG_INTERVAL = 3600;	# to reduce needless log msgs in log_tb
 our $ZERO_SLEEP_LOG_LAST = 0;
+our $WORKER_MAX_TTL_WAIT_INTERVAL = 20;		# for WORKER_MAX_TTL functionality
 
 our $WORKER_PROCESS = 0;				# used to indicate process has become a worker process
 										# this is used in addition to getppid() to prevent workers
 										# from becoming daemons in case of database instability
 
+our $WORKER_BLITZ_FACTOR = 1;			# used to determine how many workers to launch
+our $HELIOS_DB_CONN;					# database handle to help with DBI+forking issues
+
 # print help if asked
 if ( !defined($CLASS) || ($CLASS eq '--help') || ($CLASS eq '-h') ) {
 	require Pod::Usage;
@@ -265,15 +272,21 @@ print "Helios ",$Helios::VERSION,"\n";
 print "helios.pl Service Daemon version $VERSION\n";
 # --version support
 if ( $CLASS eq '--version') { exit(); }
-if ( $worker_class !~ /^[A-Za-z]([A-Za-z0-9_\-]|:{2})*[A-Za-z0-9_\-]$/ ) {
-	print "Sorry, requested name is invalid: $worker_class\n";
+#if ( $worker_class !~ /^[A-Za-z]([A-Za-z0-9_\-]|:{2})*[A-Za-z0-9_\-]$/ ) {
+#	print "Sorry, requested name is invalid: $worker_class\n";
+#	exit(1);
+#}
+print "Attempting to load $worker_class...\n"; 
+#unless ( $worker_class->can('new') ) {
+#        eval "require $worker_class";
+#        die $@ if $@;
+#}
+unless ( $worker_class ) {
+	print "You must specify the module name of a Helios service.\n";
 	exit(1);
 }
-print "Attempting to load $worker_class...\n"; 
-unless ( $worker_class->can('new') ) {
-        eval "require $worker_class";
-        die $@ if $@;
-}
+require_module($worker_class);
+
 if ( defined($worker_class->VERSION) ) {
 	print $worker_class, ' ', $worker_class->VERSION," loaded.\n";
 } else{
@@ -327,10 +340,27 @@ if ( defined($params->{registration_interval}) ) {
 } else {
 	$REGISTRATION_INTERVAL = $DEFAULTS{REGISTRATION_INTERVAL};
 }
+if ( defined($params->{WORKER_BLITZ_FACTOR}) ) {
+	$WORKER_BLITZ_FACTOR = $params->{WORKER_BLITZ_FACTOR};
+} else {
+	$WORKER_BLITZ_FACTOR = $DEFAULTS{WORKER_BLITZ_FACTOR};
+}
+if ( defined($params->{WORKER_MAX_TTL_WAIT_INTERVAL}) ) {
+	$WORKER_MAX_TTL_WAIT_INTERVAL = $params->{WORKER_MAX_TTL_WAIT_INTERVAL};
+} else {
+	$WORKER_MAX_TTL_WAIT_INTERVAL = $DEFAULTS{WORKER_MAX_TTL_WAIT_INTERVAL};	
+}
+
+# make a globally accessible database handle 
+# to make it easier to clean up CachedKids after a fork()
+$HELIOS_DB_CONN = $worker->dbConnect();
 
 if ($DEBUG_MODE) { 
 	print "MASTER LAUNCH INTERVAL: $MASTER_LAUNCH_INTERVAL\n"; 
 	print "ZERO LAUNCH INTERVAL: $ZERO_LAUNCH_INTERVAL\n"; 
+	print "ZERO SLEEP_INTERVAL: $ZERO_SLEEP_INTERVAL\n";	
+	print "REGISTRATION_INTERVAL: $REGISTRATION_INTERVAL\n";
+	print "WORKER BLITZ FACTOR: $WORKER_BLITZ_FACTOR\n";
 }
 
 my %workers;
@@ -357,8 +387,7 @@ my $times_sleeping = 0;
 # daemonize and launch the main loop
 if ( defined($ARGV[0]) && lc($ARGV[0]) eq '--clear-halt' ) {
 	clear_halt();
-	$worker->getConfigFromIni();
-	$worker->getConfigFromDb();
+	$worker->prep();
 	$params = $worker->getConfig();
 }
 if ( defined($params->{HALT}) ) {
@@ -441,10 +470,8 @@ MAIN_LOOP:{
 		
 				# recheck db parameters
 				$params = undef;
-				$worker->getConfigFromDb();	
-				$params = $worker->getConfig();
-
-
+				$params = Helios::Config->parseConfig();
+				
 				# DAEMON REGISTRATION
 				# every $REGISTRATION_INTERVAL seconds, (re)register this daemon in the database
 				if ( ($REGISTRATION_LAST + $REGISTRATION_INTERVAL) < time() ) {
@@ -504,9 +531,11 @@ MAIN_LOOP:{
 				# if we've got to this part of the loop, we have jobs that need workers to launch
 				# (though we still may not do it if we've already reached our limit)
 				my $workers_to_launch = $max_workers - $running_workers;
-				# if the number of waiting jobs is less than max workers
+				# if the number of waiting jobs is less than (max workers * worker blitz factor)
 				# then only launch one worker to reduce worker contention
-				if ( ($workers_to_launch > 0) && ($waiting_jobs < $max_workers) ) {
+				# refresh worker blitz factor, then determine if we blitz or not
+				$WORKER_BLITZ_FACTOR = $params->{WORKER_BLITZ_FACTOR} ? $params->{WORKER_BLITZ_FACTOR} : $DEFAULTS{WORKER_BLITZ_FACTOR};
+				if ( ($workers_to_launch > 0) && ($waiting_jobs < ($max_workers * $WORKER_BLITZ_FACTOR ) ) ) {
 					$workers_to_launch = 1;
 				}
 				$worker->logMsg(LOG_NOTICE, "$waiting_jobs jobs waiting; $running_workers workers running; launching $workers_to_launch workers");
@@ -521,6 +550,25 @@ MAIN_LOOP:{
 								} elsif (defined $pid) { # $pid is zero here if defined
 										# I'm the child!
 										$WORKER_PROCESS = 1;
+										
+# BEGIN CODE Copyright (C) 2012 by Logical Helion, LLC.
+										# BEFORE WE LAUNCH THE WORKER,
+										# clean up the database connections from the parent
+										# we'll set InactiveDestroy on the existing connections
+										# then we'll clear them, leaving the parent connections
+										# free of any influence of the children
+										# NOTE:  with DBI >= 1.614, all we'd have to do is
+										# set AutoInactiveDestroy on all the connections
+										# but to support the DBI (1.52) bundled with RHEL & CentOS 5,
+										# we have to make do with what we have
+										my $ck = $HELIOS_DB_CONN->{Driver}->{CachedKids};
+										foreach (keys %$ck) {
+											$ck->{$_}->{InactiveDestroy} = 1;
+										}
+										%$ck = ();
+# END CODE Copyright (C) 2012 by Logical Helion, LLC.
+
+										# NOW, launch the worker
 										launch_worker();
 							} elsif ($! == EAGAIN) {
 								# EAGAIN is the supposedly recoverable fork error
@@ -647,6 +695,7 @@ sub daemonize {
     # but I'll put them in for perlipc's sake anyway
     open STDIN, '/dev/null';
     open STDOUT, '>/dev/null';
+    open STDERR, '>/dev/null';
     
 	my $pid = fork;   
 	# make sure fork was successful
@@ -671,9 +720,10 @@ sub daemonize {
 	# session and making our own process group
 	# as well as closing any standard open filehandles.
 	POSIX::setsid();
-	close (STDIN); 
-	close (STDOUT); 
-	close (STDERR);
+#[] I may have changed religions on this one
+#	close (STDIN); 
+#	close (STDOUT); 
+#	close (STDERR);
 
 	# set up signal handling in main process 
 }
@@ -692,9 +742,8 @@ service's WORKER_MAX_TTL, the worker is killed (by sending it a SIGKILL signal).
 =cut
 
 sub double_clutch {
-    # sleep $ZERO_LAUNCH_INTERVAL secs before we double check on workers
-    sleep $ZERO_LAUNCH_INTERVAL;
-    sleep $ZERO_LAUNCH_INTERVAL;
+    # sleep $WORKER_MAX_TTL_WAIT_INTERVAL secs before we double check on workers
+	sleep $WORKER_MAX_TTL_WAIT_INTERVAL;
     foreach my $pid (keys %workers) {
         my $time_of_death = $workers{$pid} + $params->{WORKER_MAX_TTL};
         if ( time() > $time_of_death ) {
@@ -723,7 +772,7 @@ sub launch_worker {
     # just in case this would cause a problem in worker process
     $SIG{CHLD} = 'DEFAULT';
     $SIG{TERM} = 'DEFAULT';
-	my $client = TheSchwartz->new(databases => $DATABASES_INFO);
+	my $client = Helios::TheSchwartz->new(databases => $DATABASES_INFO);
 	$client->can_do($worker_class);
 	my $return;
 	if ( defined($params->{OVERDRIVE}) && $params->{OVERDRIVE} == 1 ) {
@@ -1063,6 +1112,36 @@ sub clear_halt {
 	return 1;	
 }
 
+# BEGIN CODE Copyright (C) 2012 by Logical Helion, LLC.
+
+sub require_module {
+	my $service_class = shift;
+	
+	if ( $service_class !~ /^[A-Za-z]([A-Za-z0-9_\-]|:{2})*[A-Za-z0-9_\-]$/ ) {
+		print STDERR "Sorry, requested name is invalid: $service_class\n";
+		exit(1);
+	}
+	unless ( $service_class->can('new') ) {
+        eval {
+        	my $class_file = $service_class;
+        	$class_file .= '.pm';
+        	$class_file =~ s/::/\//g;
+			require $class_file;
+			1;
+		} or do {
+			my $E = $@;
+			print STDERR "Perl module $service_class could not be loaded: $E\n";
+			exit(1);			
+		};
+	}
+	unless ($service_class->isa('Helios::Service')) {
+		print STDERR "$service_class is not a subclass of Helios::Service.\n";
+		exit(1);
+	}
+	return 1;
+}
+
+# END CODE Copyright (C) 2012 by Logical Helion, LLC.
 
 
 =head1 SEE ALSO
@@ -1077,6 +1156,9 @@ Andrew Johnson, E<lt>lajandy at cpan dotorgE<gt>
 
 Copyright (C) 2007-9 by CEB Toolbox, Inc.
 
+Portions of this software, where noted, are
+Copyright (C) 2012 by Logical Helion, LLC.
+
 This program is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself, either Perl version 5.8.0 or,
 at your option, any later version of Perl 5 you may have available.
