@@ -6,19 +6,18 @@ use warnings;
 use base qw( TheSchwartz::Worker );
 use File::Spec;
 use Sys::Hostname;
-use Config::IniFiles;
 use DBI;
-use Data::ObjectDriver::Driver::DBI;
-use Error qw(:try);
+use Helios::ObjectDriver::DBI;
 require XML::Simple;
 
 use Helios::Error;
 use Helios::Job;
+use Helios::Config;
 use Helios::ConfigParam;
 use Helios::LogEntry;
 use Helios::LogEntry::Levels qw(:all);
 
-our $VERSION = '2.41';
+our $VERSION = '2.60';
 
 =head1 NAME
 
@@ -39,7 +38,7 @@ failedJob(), or failedJobPermanent(), respectively) before it ends.
 =head1 TheSchwartz HANDLING METHODS
 
 The following 3 methods are used by the underlying TheSchwartz job queuing 
-system to determine what work is to be performed and, a job fails, how it 
+system to determine what work is to be performed and, if a job fails, how it 
 should be retried.
 
 YOU DO NOT NEED TO TOUCH THESE METHODS TO CREATE HELIOS SERVICES.  These 
@@ -79,12 +78,6 @@ enabled and the service hasn't been HALTed or told to HOLD, the worker process
 will stay running, and work() will be called to setup and run another job.  If 
 the service is not in OVERDRIVE mode, the worker process will exit.
 
-=head3 COPYRIGHT
-
-Portions of this method, where noted, are 
-Copyright (C) 2011-2012 by Andrew Johnson.  See the COPYRIGHT AND LICENSE section 
-elsewhere in this document for specific terms.
-
 =cut
 
 our $CACHED_CONFIG;
@@ -92,6 +85,7 @@ our $CACHED_CONFIG_RETRIEVAL_COUNT = 0;
 our $WORKER_START_TIME = 0;
 
 our %INIT_LOG_CLASSES;	# for the logging system
+our $INIT_CONFIG_CLASS; # for config system
 
 our $DRIVER;	# for caching the Data::ObjectDriver
 
@@ -455,8 +449,13 @@ sub prep {
 		$self->setConfig($cached_config);
 		return 1;        
     } else {
-		$self->getConfigFromIni();
-		$self->getConfigFromDb();
+		# initialize config module if it isn't already initialized
+		unless ($INIT_CONFIG_CLASS) {
+			$INIT_CONFIG_CLASS = $self->initConfig();
+		}
+		my $conf = $INIT_CONFIG_CLASS->parseConfig();
+
+		$self->setConfig($conf);
 	}
 
 	# use the given D::OD driver if we were given one
@@ -476,7 +475,7 @@ sub prep {
 }
 # END Code Copyright Andrew Johnson.
 
-=head2 getConfigFromIni([$inifile])
+=head2 getConfigFromIni([$inifile]) DEPRECATED
 
 The getConfigFromIni() method opens the helios.ini file, grabs global params and config params relevant to
 the current service class, and returns them in a hash to the calling routine.  It also sets the class's 
@@ -491,52 +490,22 @@ you, and is the preferred method.
 sub getConfigFromIni {
 	my $self = shift;
 	my $inifile = shift;
-	my $jobtype = $self->getJobType();
-	my %params;
 
-	# use the object's INI file if we weren't given one explicitly
-	# or use the contents of the HELIOS_INI env var
-	unless ($inifile) {
-		if ( $self->getIniFile() ) {
-			$inifile = $self->getIniFile();
-		} elsif ( defined($ENV{HELIOS_INI}) ) {
-			$inifile = $ENV{HELIOS_INI};
-		} elsif (-r File::Spec->catfile(File::Spec->curdir,'helios.ini') ) {
-			$inifile = File::Spec->catfile(File::Spec->curdir,'helios.ini');
-		} else {
-			throw Helios::Error::InvalidArg("INI configuration file not specified.");
-		}
-	}
-
-	if ( $self->debug() ) { print "inifile: $inifile\nclass:$jobtype\n"; }		
-	unless (-r $inifile) { $self->errstr("INI read error: $!"); return undef; }
-
-	my $ini = new Config::IniFiles( -file => $inifile );
-	unless ( defined($ini) ) { throw Helios::Error::Fatal("Invalid INI file; check configuration"); }
-
-	# global must exist; it's where the helios db is declared
-	if ($ini->SectionExists("global") ) {
-		foreach ( $ini->Parameters("global") ) {
-			$params{$_} = $ini->val("global", $_);
-		}
-	} else {
-		throw Helios::Error::InvalidArg("Section [global] doesn't exist in config file $inifile");
-	}
+# BEGIN CODE Copyright (C) 2012 by Logical Helion, LLC.
 	
-	# if there's a section specifically for this service class, read it too
-	# (it will effectively override the global section, BTW)
-	if ( $ini->SectionExists($jobtype) ) {
-		foreach ( $ini->Parameters($jobtype) ) {
-			$params{$_} = $ini->val($jobtype, $_);
-		}
+	unless ($INIT_CONFIG_CLASS) {
+		if ( defined($inifile) ) { $self->setIniFile($inifile); }
+		$INIT_CONFIG_CLASS = $self->initConfig();
 	}
+	my $conf = $INIT_CONFIG_CLASS->parseConfFile();
+	$self->setConfig($conf);
+	return %{$conf};
+# END CODE Copyright (C) 2012 by Logical Helion, LLC.
 
-	$self->setConfig(\%params);
-	return %params;
 }
 
 
-=head2 getConfigFromDb()
+=head2 getConfigFromDb() DEPRECATED
 
 The getConfigFromDb() method connects to the Helios database, retrieves config params relevant to the 
 current service class, and returns them in a hash to the calling routine.  It also sets the class's 
@@ -558,50 +527,34 @@ will take care of things for the most part.
 sub getConfigFromDb {
 	my $self = shift;
 	my $params = $self->getConfig();
-	my $hostname = $self->getHostname();
-	my $jobtype = $self->getJobType();
-	my @cps;
-	my $cp;
 
-	if ($self->debug) { print "Retrieving params for ".$self->getJobType()." on ".$self->getHostname()."\n"; }
-
-	try {
-		my $driver = $self->getDriver();
-		@cps = $driver->search('Helios::ConfigParam' => {
-				worker_class => $jobtype,
-				host         => '*',
-			}
-		);
-		foreach $cp (@cps) {
-			if ($self->debug) { 
-				print $cp->param(),'=',$cp->value(),"\n";
-			}
-			$params->{$cp->param()} = $cp->value();
-		}
-		@cps = $driver->search('Helios::ConfigParam' => {
-				worker_class => $jobtype,
-				host         => $hostname,
-			}
-		);
-		foreach $cp (@cps) {
-			if ($self->debug) { 
-				print $cp->param(),'=',$cp->value(),"\n";
-			}
-			$params->{$cp->param()} = $cp->value();
-		}
-		
-	} otherwise {
-		my $e = shift;
-		throw Helios::Error::DatabaseError($e->text);
-	};
+# BEGIN CODE Copyright (C) 2012 by Logical Helion, LLC.
 
+	unless ($INIT_CONFIG_CLASS) {
+		$INIT_CONFIG_CLASS = $self->initConfig();
+	}
+	my $dbconf = $INIT_CONFIG_CLASS->parseConfDb();
+	while (my ($key, $value) = each %$dbconf ) {
+		$params->{$key} = $value;
+	}
 	$self->setConfig($params);
 	return %{$params};
+# END CODE Copyright (C) 2012 by Logical Helion, LLC.
+
 }
 
 
 =head2 getFuncidFromDb()
 
+Queries the collective database for the funcid of the service class and 
+returns it to the calling routine.  The service name used in the query is the 
+value returned from the getJobType() accessor method.  
+
+This method is most commonly used by helios.pl to get the funcid associated 
+with a particular service class, so it can scan the job table for waiting jobs.
+If their are jobs for the service waiting, helios.pl may launch new worker 
+processes to perform these jobs.
+
 =cut
 
 sub getFuncidFromDb {
@@ -612,22 +565,20 @@ sub getFuncidFromDb {
 
     if ($self->debug) { print "Retrieving funcid for ".$self->getJobType()."\n"; }
 
-    try {
-        my $driver = $self->getDriver();
-        # also get the funcid 
-        my @funcids = $driver->search('TheSchwartz::FuncMap' => {
-                funcname => $jobtype 
-            }
-        );
-        if ( scalar(@funcids) > 0 ) {
-            $self->setFuncid( $funcids[0]->funcid() );
-        }
-		
-	} otherwise {
-		my $e = shift;
-		throw Helios::Error::DatabaseError($e->text);
+	eval {
+		my $driver = $self->getDriver();
+		# also get the funcid 
+		my @funcids = $driver->search('TheSchwartz::FuncMap' => { funcname => $jobtype });
+		if ( scalar(@funcids) > 0 ) {
+			$self->setFuncid( $funcids[0]->funcid() );
+		}
+		1;
+	} or do {
+		my $E = $@;
+		Helios::Error::DatabaseError->throw("$E");
 	};
-    return $self->getFuncid();	
+
+	return $self->getFuncid();	
 }
 
 
@@ -644,50 +595,36 @@ sub jobsWaiting {
 	my $self = shift;
 	my $params = $self->getConfig();
 	my $jobType = $self->getJobType();
-	my $funcid;
 
-	try {
 
-		my $dbh = $self->dbConnect($params->{dsn}, $params->{user}, $params->{password});
-		unless ($dbh) { throw Helios::Error::DatabaseError($self->errstr); }
+# BEGIN CODE Copyright (C) 2012 by Logical Helion, LLC.
 
-		# get the funcid if we don't already have it
-		if ( !defined($self->getFuncid()) ) {
-		    $funcid = $self->getFuncidFromDb();
-		} else {
-		    $funcid = $self->getFuncid();
+	my $jobsWaiting;
+	my $funcid = $self->getFuncid();
+	eval {
+		my $dbh = $self->dbConnect();
+		unless ( defined($funcid) ) {
+			$funcid = $self->getFuncidFromDb();
 		}
-
-#		my $sql2 = <<JWSQL2;
-#SELECT COUNT(*)
-#FROM job
-#WHERE funcid = ?
-#	AND (run_after < ?)
-#	AND (grabbed_until < ?)
-#JWSQL2
-		my $sql2 = qq{
-SELECT COUNT(*)
-FROM job
-WHERE funcid = ?
-	AND (run_after < ?)
-	AND (grabbed_until < ?)
-		};
-		my $sth2 = $dbh->prepare($sql2);
-
-		my $current_time = time();
-		$sth2->execute($funcid, $current_time, $current_time);
-		my $result2 = $sth2->fetchrow_arrayref();
-		unless ( defined($result2->[0]) ) {
-			$self->errstr("Received NULL value in jobsWaiting() for funcname $jobType!"); return undef; 
-		}
-		$sth2->finish();
-		$dbh->disconnect();
-		return $result2->[0];
-
-	} otherwise {
-		throw Helios::Error::DatabaseError($DBI::errstr);
+		
+		my $sth = $dbh->prepare_cached('SELECT COUNT(*) FROM job WHERE funcid = ? AND (run_after < ?) AND (grabbed_until < ?)');
+		$sth->execute($funcid, time(), time());
+		my $r = $sth->fetchrow_arrayref();
+		$sth->finish();
+		$jobsWaiting = $r->[0];
+		
+		1;
+	} or do {
+		my $E = $@;
+		Helios::Error::DatabaseError->throw("$E");
 	};
 	
+	return $jobsWaiting;
+# END CODE Copyright (C) 2012 by Logical Helion, LLC.
+
+
+
+
 }
 
 
@@ -708,7 +645,7 @@ sub initDriver {
 	my $self = shift;
 	my $config = $self->getConfig();
 	if ($self->debug) { print $config->{dsn},$config->{user},$config->{password},"\n"; }
-	my $driver = Data::ObjectDriver::Driver::DBI->new(
+	my $driver = Helios::ObjectDriver::DBI->new(
 	    dsn      => $config->{dsn},
 	    username => $config->{user},
 	    password => $config->{password}
@@ -744,18 +681,32 @@ sub shouldExitOverdrive {
 }
 
 
+
 =head1 METHODS AVAILABLE TO SERVICE SUBCLASSES
 
 The methods in this section are available for use by Helios services.  They 
 allow your service to interact with the Helios environment.
 
-=head2 dbConnect($dsn, $user, $password)
+=cut
 
-Method to connect to a database.  If parameters not specified, uses dsn, user, password 
-from %params hash (the Helios database).
+# BEGIN CODE Copyright (C) 2012 by Logical Helion, LLC.
 
-This method uses the DBI->connect_cached() method to attempt to reduce the number of open 
-connections to a particular database.
+=head2 dbConnect($dsn, $user, $password, $options)
+
+Method to connect to a database in a "safe" way.  If the connection parameters 
+are not specified, a connection to the Helios collective database will be 
+returned.  If a connection to the given database already exists, dbConnect() 
+will return a database handle to the existing connection rather than create a 
+new connection.
+
+The dbConnect() method uses the DBI->connect_cached() method to reuse database 
+connections and thus reduce open connections to your database (often important
+when you potentially have hundreds of active worker processes working in a 
+Helios collective).  It "tags" the connections it creates with the current PID 
+to prevent reusing a connection that was established by a parent process.  
+That, combined with helios.pl clearing connections after the fork() to create 
+a worker process, should allow for safe database connection/disconnection in 
+a forking environment.
 
 =cut
 
@@ -766,39 +717,71 @@ sub dbConnect {
 	my $password = shift;
 	my $options = shift;
 	my $params = $self->getConfig();
+	my $connect_to_heliosdb = 0;
 
-	# if we weren't given params
-	unless ($dsn) {
+	# if we weren't given params, 
+	# we'll default to the Helios collective database
+	unless ( defined($dsn) ) {
 		$dsn = $params->{dsn};
 		$user = $params->{user};
 		$password = $params->{password};
 		$options = $params->{options};
+		$connect_to_heliosdb = 1;
 	}
 
-	try {
+	my $dbh;
+	my $o;
 
-		my $dbh;
+	eval {
+
+		# if we were given options, parse them into a hashref
+		# throw a config error if this fails
 		if ($options) {
-			my $o = eval "{$options}";
-			if ($@) { $self->errstr($@); return undef;	}
-			if ($self->debug) { print "dsn=$dsn\nuser=$user\npass=$password\noptions=$options\n"; }	
-			$dbh = DBI->connect_cached($dsn, $user, $password, $o);	
-		} else {
-			if ($self->debug) { print "dsn=$dsn\nuser=$user\npass=$password\n";	} 
-			$dbh = DBI->connect_cached($dsn, $user, $password);
+			$o = eval "{$options}";
+			Helios::Error::ConfigError->throw($@) if $@;
 		}
-        unless ( defined($dbh) ) {
-			$self->errstr("DB ERROR: ".$DBI::errstr); 
-			throw Helios::Error::DatabaseError($DBI::errstr);
+		
+		# if we're connecting to the collective db, 
+		# we _must_ force certain options to make sure the "new" connection
+		# doesn't disrupt Helios operations
+		# (Previous dbConnect() code didn't properly handle connection creation
+		#  because it effectively ignored the "options" config param
+		if ( $connect_to_heliosdb ) {
+			$o->{RaiseError} = 1;
+			$o->{AutoCommit} = 1;
 		}
-		$dbh->{RaiseError} = 1;
-		return $dbh;
+		# ALL db connections created by dbConnect() get a "tag" 
+		# this is to generally make sure if a fork has happened, 
+		# we don't allow DBI to reuse a connection the parent made
+		# (helios.pl should be clearing those now, though)
+		$o->{'private_heliconn_dbconnect_'.$$} = $$;
+		
+		# debug
+		if ($self->debug) { 
+			print "dbConnect():\n\tdsn=$dsn\n";
+			if ( defined($user)   ) { print "\tuser=$user\n"; }
+			if ( defined($options)) { print "\toptions=$options\n"; } 
+		}	
 
-	} otherwise {
-		throw Helios::Error::DatabaseError($DBI::errstr);
+		# make the connection!
+		$dbh = DBI->connect_cached($dsn, $user, $password, $o);	
+
+		# if we *didn't* get a database connection, we have to throw an error
+		unless ( defined($dbh) ) {
+			Helios::Error::DatabaseError->throw($DBI::errstr);
+		}
+
+		1;
+	} or do {
+		# whatever exception was thrown, 
+		# we're going to cast it into a DatabaseError
+		my $E = $@;
+		Helios::Error::DatabaseError->throw("$E");
 	};
-
+	
+	return $dbh;
 }
+# END CODE Copyright (C) 2012 by Logical Helion, LLC.
 
 
 =head2 logMsg([$job,] [$priority_level,] $message)
@@ -900,20 +883,10 @@ this method will attempt to catch the error and log it to the
 Helios::Logger::Internal internal logger.  It will then rethrow the error 
 as a Helios::Error::LoggingError exception.
 
-=head3 COPYRIGHT AND LICENSE
-
-The logMsg() method is Copyright (C) 2009-12 by Andrew Johnson
-
-This method is free software; you can redistribute it and/or modify
-it under the same terms as Perl itself, either Perl version 5.8.0 or,
-at your option, any later version of Perl 5 you may have available.
-
-=head3 WARRANTY
-
-This software comes with no warranty of any kind.
-
 =cut
 
+# BEGIN CODE Copyright (C) 2009-12 by Andrew Johnson.
+
 sub logMsg {
 	my $self = shift;
 	my @args = @_;
@@ -922,6 +895,7 @@ sub logMsg {
 	my $msg;
 	my @loggers;
 
+
 	# were we called with 3 params?  ($job, $level, $msg)
 	# 2 params?                      ($level, $msg) or ($job, $msg)
 	# or just 1?                     ($msg)
@@ -966,6 +940,58 @@ sub logMsg {
 	
 	return 1;	
 }
+# END CODE Copyright (C) 2009-12 by Andrew Johnson.
+
+
+# BEGIN CODE Copyright (C) 2012 by Logical Helion, LLC.
+
+=head2 initConfig()
+
+The initConfig() method is called to initialize the configuration parsing 
+class.  This method is normally called by the prep() method before a service's 
+run() method is called; most Helios application developers do not need to 
+worry about this method.
+
+The normal Helios config parsing class is Helios::Config.  This can be 
+changed by specifying another config class with the ConfigClass() method in 
+your service.
+
+This method will throw a Helios::Error::ConfigError if anything goes wrong 
+with config class initialization.
+
+=cut
+
+sub initConfig {
+	my $self = shift;
+	my $config_class = $self->ConfigClass() ? $self->ConfigClass() : 'Helios::Config';
+	
+	# only initialize the config system once
+	unless( defined($INIT_CONFIG_CLASS) ) {
+
+#		if ( $config_class !~ /^[A-Za-z]([A-Za-z0-9_\-]|:{2})*[A-Za-z0-9_\-]$/ ) {
+#			Helios::Error::ConfigError->throw("Requested Config class name is invalid: ".$config_class);
+#		}
+#
+#		# attempt class load if it hasn't been already
+#		unless ( $config_class->can('init') ) {
+#			eval "require $config_class";
+#		    Helios::Error::ConfigError->throw($@) if $@;
+#		}
+
+		$self->_require_module($config_class, 'Helios::Config');
+		
+		$config_class->init(
+			CONF_FILE => $self->getIniFile(),
+			SERVICE   => $self->getJobType(),
+			HOSTNAME  => $self->getHostname(),
+			DEBUG     => $self->debug()
+		);
+		$INIT_CONFIG_CLASS = $config_class;
+	}
+	return $config_class;
+}
+
+# END CODE Copyright (C) 2012 by Logical Helion, LLC.
 
 
 =head2 initLoggers()
@@ -1011,14 +1037,15 @@ sub initLoggers {
 	foreach my $logger (@loggers) {
 		# init the logger if it hasn't been initialized yet
 		unless ( defined($INIT_LOG_CLASSES{$logger}) ) {
-			if ( $logger !~ /^[A-Za-z]([A-Za-z0-9_\-]|:{2})*[A-Za-z0-9_\-]$/ ) {
-				Helios::Error::LoggingError->throw("Sorry, requested Logger name is invalid: ".$logger);
-			}
-			# attempt to init the class
-			unless ( $logger->can('init') ) {
-		        eval "require $logger";
-		        throw Helios::Error::LoggingError($@) if $@;
-			}
+#			if ( $logger !~ /^[A-Za-z]([A-Za-z0-9_\-]|:{2})*[A-Za-z0-9_\-]$/ ) {
+#				Helios::Error::LoggingError->throw("Sorry, requested Logger name is invalid: ".$logger);
+#			}
+#			# attempt to init the class
+#			unless ( $logger->can('init') ) {
+#		        eval "require $logger";
+#		        throw Helios::Error::LoggingError($@) if $@;
+#			}
+			$self->_require_module($logger,'Helios::Logger');
 			$logger->setConfig($config);
 			$logger->setJobType($jobType);
 			$logger->setHostname($hostname);
@@ -1190,14 +1217,68 @@ the default.
 sub JobClass { return undef; }
 
 
+# BEGIN CODE Copyright (C) 2012 by Logical Helion, LLC.
+
+=head2 ConfigClass()
+
+Defines which configuration class to use to parse your service's 
+configuration.  The default is Helios::Config, which should work fine for most 
+applications.  If necessary, you can create a subclass of Helios::Config and 
+set your ConfigClass() method to return that subclass's name.  The service's 
+prep() method will initialize your custom config class and use it to parse your 
+service's configuration information.
+
+See the L<Helios::Config> documentation for more information about creating 
+custom config classes.
+
+=cut
+
+sub ConfigClass { return undef; }
+
+# END CODE Copyright (C) 2012 by Logical Helion, LLC.
+
+
+# BEGIN CODE Copyright (C) 2012 by Logical Helion, LLC.
+
+sub _require_module {
+	my $self = shift;
+	my $class = shift;
+	my $requested_superclass = shift;
+	
+	if ( $class !~ /^[A-Za-z]([A-Za-z0-9_\-]|:{2})*[A-Za-z0-9_\-]$/ ) {
+		Helios::Error::ConfigError->throw("Requested module name is invalid: $class");
+	}
+	unless ( $class->can('init') ) {
+        eval {
+        	my $class_file = $class;
+        	$class_file .= '.pm';
+        	$class_file =~ s/::/\//g;
+			require $class_file;
+			1;
+		} or do {
+			my $E = $@;
+			Helios::Error::ConfigError->throw("Requested module $class could not be loaded: $E");
+		};
+	}
+	if ($requested_superclass && !$class->isa($requested_superclass)) {
+		Helios::Error::ConfigError->throw("$class is not a subclass of $requested_superclass.");
+	}
+	return 1;
+}
+
+# END CODE Copyright (C) 2012 by Logical Helion, LLC.
+
+
+
+
 1;
 __END__
 
 
 =head1 SEE ALSO
 
-L<helios.pl>, L<Helios::Job>, L<Helios::Error>, L<Helios::ConfigParam>, L<Helios::LogEntry>, 
-L<TheSchwartz>, L<XML::Simple>, L<Config::IniFiles>, L<Sys::Syslog>
+L<Helios>, L<helios.pl>, L<Helios::Job>, L<Helios::Error>, L<Helios::Config>, 
+L<TheSchwartz>
 
 =head1 AUTHOR
 
@@ -1213,6 +1294,9 @@ Copyright (C) 2009 by Andrew Johnson.
 Portions of this software, where noted, are
 Copyright (C) 2011-2012 by Andrew Johnson.
 
+Portions of this software, where noted, are
+Copyright (C) 2012 by Logical Helion, LLC.
+
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself, either Perl version 5.8.0 or,
 at your option, any later version of Perl 5 you may have available.
